#!/usr/bin/perl
use strict;
use DBI::DBD;

# ignore terminated or stopped child processes
$SIG{'CHLD'} = 'IGNORE';

# disable output buffering
$| = 1;

my $debug = 0;

my $dbfile = "/path/to/streamer.db";
my $command = "/path/to/darkice";
my $pid;

sub msg {
	my $msg = shift;
	my $stamp = localtime;
	printf "%s %s\n", $stamp, $msg;
}

sub fork_and_run {
	my $pid = fork();
	
	if (not defined $pid) {
		msg "resources not available";
	
	} elsif ($pid == 0) {
		msg "[child] executing: $command";
		exec $command;
	
	} else {
		msg "[parent] child forked: $pid";
		return $pid;
	}
}

sub cleanup {
	msg "[parent] cleaning up";
	if (defined($pid) && kill(0, $pid)) {
		kill(15, $pid);
	}
	exit;
}

msg "[parent] started";

$SIG{'INT'} = \&cleanup;
$SIG{'TERM'} = \&cleanup;

while(1) {
	my $now = time;
	my $runit;

	# db check: should there be a stream and is it running (running == true?)
	my $db = DBI->connect("dbi:SQLite2:dbname=$dbfile") or die "db connect failed";
	my $sql = $db->prepare("SELECT streamid, start, running, due, finished FROM stream WHERE (start < $now AND due > $now)");
	$sql->execute();
	my($streamid, $start, $running, $due, $finished);
	$sql->bind_columns( undef, \$streamid, \$start, \$running, \$due, \$finished );
	$sql->fetch();
	my $rows = $sql->rows;

	if ($rows == -1) {
		msg "[parent] db: stream sould NOT be running" unless $debug == 0;
		$runit = 'no';
	} elsif ($rows == 1) {
		msg "[parent] db: stream $streamid sould be running" unless $debug == 0;
		$runit = 'yes';
	}

	$sql->finish();
	$db->disconnect();

	# live: is the stream running (^^ and should it be running?)
	if (defined($pid) && kill(0, $pid)) {
		msg "[parent] Stream is running (pid $pid)" unless $debug == 0;
		# stop stream
		if ($runit eq 'no') {
			msg "[parent] stopping stream";
			kill(15, $pid);

			my $db = DBI->connect("dbi:SQLite2:dbname=$dbfile") or die "db connect failed";
			my $sql = $db->prepare("UPDATE stream SET running='false', finished='true' where running='true'");
			$sql->execute();
			$db->disconnect();
		}
	} else {
		# start stream
		if($runit eq 'yes') {
			msg "[parent] starting stream $streamid";
			$pid = fork_and_run();

			my $db = DBI->connect("dbi:SQLite2:dbname=$dbfile") or die "db connect failed";
			my $sql = $db->prepare("UPDATE stream SET running='true' where streamid=$streamid");
			$sql->execute();
			$db->disconnect();
		}
	}
	
	sleep 20;
}
